----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
----------------------------------------------------------------------------------
-- This IO-Module will map the IO space of a Picoblaze (256 inputs and 256 outputs
-- of 8-bit) to a mix of input and output devices.
-- The way's its done surely not optimal in order to save logic, but its easy
-- Please note that DSIO definition applies that data can be written and read to/from
-- the same address.
entity IO_Module2 is
    Port ( Clk :       in  STD_LOGIC;
			  out_port :  in  STD_LOGIC_VECTOR (7 downto 0);
           in_port :   out STD_LOGIC_VECTOR (7 downto 0);
			  Write_strobe,
			  Read_strobe:    STD_LOGIC;
			  Port_ID :   in  STD_LOGIC_VECTOR (7 downto 0);
			  ------------------------------------------------
			  VGA_Plug:   inout STD_LOGIC_VECTOR (9 downto 0);
			  ------------------------------------------------	  
			  PS2_Clk:    in  STD_LOGIC;
			  PS2_Data:   in  STD_LOGIC;
			  ------------------------------------------------			  
           Segm :      out STD_LOGIC_VECTOR (1 to 8);
           An :        out STD_LOGIC_VECTOR (3 downto 0);
           Led :       out STD_LOGIC_VECTOR (7 downto 0);
			  T_Scale:    out STD_LOGIC_VECTOR (7 downto 0);   -- Scale for Timer
           Sw :        in  STD_LOGIC_VECTOR (7 downto 0);
			  Btn :       in  STD_LOGIC_VECTOR (3 downto 0));
end IO_Module2;

architecture Behavioral of IO_Module2 is
------------------------------------------------------------------------------	
   signal Digital_Leds:	 std_logic_vector( 7 downto 0);

---------------------------------------Display signals-----------------------
	Signal Clk_1kHz:      std_logic;
	Signal Bcd_ciffer:    std_logic_vector( 3 downto 0);
	Signal Decimal_Punkt: std_logic;
	Signal Segmenter:     std_logic_vector( 1 to 7);
	signal Bcd:           std_logic_vector(15 downto 0);
	signal Dp: 				 std_logic_vector( 3 downto 0);	
	
	COMPONENT muxdisplay_ver3
	PORT(
		Clk_50MHz : IN  std_logic;
		Bcd : 		IN  std_logic_vector(15 downto 0);
		Dp : 			IN  std_logic_vector(3 downto 0);          
		Segm : 		OUT std_logic_vector(1 to 8);
		An : 			OUT std_logic_vector(3 downto 0);
		Clk_1kHz : 	OUT std_logic
		);
	END COMPONENT;	
---------------------------------------VGA signals------------------------	
	SIGNAL Reset:       	 std_logic;
   SIGNAL Data_From_RAM: STD_LOGIC_VECTOR ( 3 DOWNTO 0);
   SIGNAL Addr:			 STD_LOGIC_VECTOR (12 DOWNTO 0); 
	SIGNAL Line:          STD_LOGIC_VECTOR (7 DOWNTO 0); 
	SIGNAL Pixel:         STD_LOGIC_VECTOR (7 DOWNTO 0); 
   SIGNAL Data_To_RAM:	 STD_LOGIC_VECTOR ( 3 DOWNTO 0); 
   SIGNAL WE:				 STD_LOGIC;
	
	SIGNAL Hcnt,Vcnt:    STD_LOGIC_VECTOR (16 DOWNTO 0);	

   COMPONENT VGA_system
   PORT(  Reset:		IN	 STD_LOGIC; 
          Clk:			IN	 STD_LOGIC; 
          Data_out:	OUT STD_LOGIC_VECTOR ( 3 DOWNTO 0); 
          VGA_Plug:	OUT STD_LOGIC_VECTOR ( 9 DOWNTO 0); 
          Addr:		IN	 STD_LOGIC_VECTOR (15 DOWNTO 0); 
          Data_in:	IN	 STD_LOGIC_VECTOR ( 3 DOWNTO 0); 
          WE:			IN	 STD_LOGIC);
   END COMPONENT;
---------------------------------------PS2 Keyboard-----------------------		
	SIGNAL DoRead :      std_logic;    
	SIGNAL Trigger :     std_logic;      
	SIGNAL Scan_Code :   std_logic_vector(7 downto 0);
	SIGNAL Scan_ready :  std_logic;

	COMPONENT ps2_ctrl_VHDL
	PORT(
		Clk : IN std_logic;
		Reset : IN std_logic;
		PS2_Clk : IN std_logic;
		PS2_Data : IN std_logic;
		DoRead : IN std_logic;    
		Trigger : INOUT std_logic;      
		Scan_Code : OUT std_logic_vector(7 downto 0);
		Scan_ready : OUT std_logic
		);
	END COMPONENT;
	
BEGIN

	Muxdisplay: muxdisplay_ver3 PORT MAP(
		Clk_50MHz=> Clk,
		Bcd => 		Bcd,
		Dp =>  		Dp,
		Segm => 		Segm ,
		An => 		An,
		Clk_1kHz => Clk_1kHz
	);

   VGA_screen: VGA_system PORT MAP(
		Reset 	=> Reset, 
		Clk 		=> Clk, 
		Data_out => Data_From_RAM, 
		VGA_Plug => VGA_Plug, 
		Addr 		=> Addr, 
		Data_in 	=> Data_To_RAM, 
		WE 		=> WE 
   );
	
	PS2_Keyboard: ps2_ctrl_VHDL PORT MAP(
		Clk 			=> Clk,
		Reset 		=> Reset,
		PS2_Clk 		=> PS2_Clk,
		PS2_Data 	=> PS2_Data,
		Scan_Code 	=> Scan_Code,
		Scan_ready 	=> Scan_ready,
		Trigger 		=> Trigger,
		DoRead 		=> DoRead 	
	);

	Addr  <= Line & Pixel;
	Reset <= '0';
   DoRead <= '1';
 
   process( VGA_Plug) -- Test purpose
   begin
      if rising_edge( VGA_Plug(8)) then
         Hcnt <= Hcnt+1;
      end if;
   end process;	
	
   process( VGA_Plug) -- Test purpose
   begin
      if rising_edge( VGA_Plug(9)) then
         Vcnt <= Vcnt+1;
      end if;
   end process;	
  ---------------------------------------------------------------------
  -- KCPSM3 input ports 
  ---------------------------------------------------------------------
  input_ports: process(clk)
  begin
     if clk'event and clk='1' then
--	     if Read_strobe='1' then
			  Case conv_integer( Port_id(5 downto 0)) is
					when  0 => in_port <= "0000"& Bcd(  3 downto  0);
					when  1 => in_port <= "0000"& Bcd(  7 downto  4);
					when  2 => in_port <= "0000"& Bcd( 11 downto  8);
					when  3 => in_port <= "0000"& Bcd( 15 downto 12);
					when  4 => in_port <= Bcd( 7 downto 0);
					when  5 => in_port <= Bcd(15 downto 8);
					when  6 => in_port <= "0000" & Dp;
					when  7 => in_port <= "00000000";
					when  9 => in_port <= Digital_Leds;			  
				  ------------------------------------------------------------
					when 20 => in_port <=  Sw;
					when 21 => in_port <=  "0000"& Btn;
				  ------------------------------------------------------------
				   when 24 => in_port <= Scan_Code;
					when 25 => in_port <= PS2_Data&PS2_Clk&"00000"&Scan_Ready;
				  ------------------------------------------------------------
					when 32 => in_port <= "0000"&Data_From_RAM;
					when 33 => in_port <= Line;
					when 34 => in_port <= Pixel;
					when 40 => in_port <= Hcnt(16 downto 9); -- Test purpose
					when 41 => in_port <= Vcnt(15 downto 8);	-- Test purpose				
					when others => null;
			  end case;
--		  end if;
	  end if;
  end process input_ports;

  -----------------------------------------------------------------------
  -- KCPSM3 output ports 
  -----------------------------------------------------------------------
  output_ports: process(clk)
  begin

     if clk'event and clk='1' then
	      WE <= '0';
			if write_strobe='1' then
				Case conv_integer( Port_id(5 downto 0)) is
					when  0 => Bcd( 3 downto  0) <= out_port(3 downto 0);
					when  1 => Bcd( 7 downto  4) <= out_port(3 downto 0);
					when  2 => Bcd(11 downto  8) <= out_port(3 downto 0);
					when  3 => Bcd(15 downto 12) <= out_port(3 downto 0);
					when  4 => Bcd( 7 downto  0) <= out_port;
					when  5 => Bcd(15 downto  8) <= out_port;
					when  6 => Dp                <= out_port(3 downto 0);
					when  7 => T_Scale   			<= out_port;

					when  9 => Led 		         <= out_port;
				 ------------------------------------------------------
					when 32 => Data_To_RAM 		   <= out_port(3 downto 0);
					           WE   <= '1'; 
					when 33 => Line 			   	<= out_port(7 downto 0);								  
					when 34 => Pixel 			   	<= out_port(7 downto 0);
					when 35 => Data_To_RAM 		   <= out_port(3 downto 0); --LF
					           WE   <= '1';	
								  IF Line>="111011" THEN 
								     Line <= (others=>'0');
								  ELSE
									  Line <= Line+1;
								  END IF; 
					when 36 => Data_To_RAM		   <= out_port(3 downto 0); --NextPixel
					           WE    <= '1';
                          IF Pixel>="1111111" THEN  --alternative "1001111"
                             Pixel <= (others=>'0');
                          ELSE									  
                             Pixel <= Pixel+1;	
								  END IF;
					when 37 => Data_To_RAM		   <= out_port(3 downto 0); -- CR/LF
					           WE 	  <= '1';	
                          IF Pixel>="1111111" THEN --alternative "1001111"
                             Pixel <= (others=>'0');
									  IF Line>="111011" THEN 
										  Line <= (others=>'0');
									  ELSE
										  Line <= Line+1;
									  END IF;									 
                          ELSE									  
                             Pixel <= Pixel+1;	
								  END IF;									  
					when others => null;
				end case;
			end if;
     end if;
  end process output_ports;

end Behavioral;

