-- Create Date:    15:19:14 03/19/2009 
-- Design Name: 
-- Module Name:    PicoBlaze_MicroComputer1 - Behavioral 
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity PicoBlaze_MicroComputer1 is
    Port ( Clk :       in  STD_LOGIC;
           Segm :      	out STD_LOGIC_VECTOR (1 to 8);
           An :        	out STD_LOGIC_VECTOR (3 downto 0);
           Led :      inout STD_LOGIC_VECTOR (7 downto 0);
           Sw :        	in  STD_LOGIC_VECTOR (7 downto 0);
			  Btn :       	in  STD_LOGIC_VECTOR (3 downto 0);
			  JA :       	in  STD_LOGIC_VECTOR (1 to 4);
			  PS2_Data:   in  STD_LOGIC;
			  ------------------------------------------------				  
			  VGA_Plug:   inout STD_LOGIC_VECTOR (9 downto 0);
			  ------------------------------------------------	  
			  PS2_Clk:    in  STD_LOGIC);
			  
end PicoBlaze_MicroComputer1;

architecture Behavioral of PicoBlaze_MicroComputer1 is
   -----------------------------------------------------------------------------
	COMPONENT Muxdisplay_ver5f
	PORT(	Clk_50MHz : IN std_logic;
			Blank : 		IN std_logic_vector(3 downto 0);
			Hex : 		IN std_logic_vector(15 downto 0);
			Dp : 			IN std_logic_vector(3 downto 0);          
			Segm : 		OUT std_logic_vector(1 to 8);
			An : 			OUT std_logic_vector(3 downto 0);
			Clk_1kHz : 	OUT std_logic;
			Clk_1Hz : 	OUT std_logic	);
	END COMPONENT;
	Signal Blank : 	std_logic_vector(3 downto 0);
	Signal Hex :  		std_logic_vector(15 downto 0);
	Signal Dp :  		std_logic_vector(1 to 4);          
	Signal Clk_1kHz : std_logic;
	Signal Clk_1Hz :  std_logic;
   -----------------------------------------------------------------------------	
	COMPONENT Timer
	PORT(	Clk : 				IN  std_logic;
			interrupt_ack : 	IN  std_logic;
			T_Scale : 			IN  std_logic_vector(7 downto 0);          
			interrupt : 		OUT std_logic);
	END COMPONENT;	
	Signal interrupt, interrupt_ack : std_logic;
	Signal T_Scale :  				     std_logic_vector(7 downto 0);          
   -----------------------------------------------------------------------------
   COMPONENT PicoBlaze_System1
   PORT( Clk	:				IN		STD_LOGIC; 
          write_strobe	:	OUT	STD_LOGIC; 
          read_strobe	:	OUT	STD_LOGIC; 
          interrupt_ack	:	OUT	STD_LOGIC; 
          interrupt	:		IN		STD_LOGIC; 
          in_port	:			IN		STD_LOGIC_VECTOR (7 DOWNTO 0); 
          port_id	:			OUT	STD_LOGIC_VECTOR (7 DOWNTO 0); 
          out_port	:		OUT	STD_LOGIC_VECTOR (7 DOWNTO 0));
   END COMPONENT;
   Signal write_strobe,read_strobe:	  STD_LOGIC; 
   Signal in_port,out_port,port_id:  STD_LOGIC_VECTOR (7 DOWNTO 0); 
---------------------------------------VGA signals------------------------	
   COMPONENT VGA_system
   PORT( Clk:			IN	STD_LOGIC; 
          Addr:		IN	STD_LOGIC_VECTOR (15 DOWNTO 0); 
          Data_in:	IN	STD_LOGIC_VECTOR (3 DOWNTO 0); 
          WE:			IN	STD_LOGIC; 
          Reset:		IN	STD_LOGIC; 
          VGA_Plug:	OUT	STD_LOGIC_VECTOR (9 DOWNTO 0); 
          Data_out:	OUT	STD_LOGIC_VECTOR (3 DOWNTO 0));
   END COMPONENT;

	SIGNAL Reset:       	 STD_LOGIC;
   SIGNAL Data_From_RAM: STD_LOGIC_VECTOR ( 3 DOWNTO 0);
   SIGNAL Addr:			 STD_LOGIC_VECTOR (15 DOWNTO 0); 
	SIGNAL Line:          STD_LOGIC_VECTOR (7 DOWNTO 0); 
	SIGNAL Pixel:         STD_LOGIC_VECTOR (7 DOWNTO 0); 
   SIGNAL Data_To_RAM:	 STD_LOGIC_VECTOR ( 3 DOWNTO 0); 
   SIGNAL WE:				 STD_LOGIC;
	-- Extra signals in order to create Line offsets
	SIGNAL Temp_line,Line_Offset: STD_LOGIC_VECTOR (7 DOWNTO 0);
	
---------------------------------------PS2 Keyboard-----------------------	
	COMPONENT ps2_ctrl_VHDL
	PORT(
		Clk : IN std_logic;
		Reset : IN std_logic;
		PS2_Clk : IN std_logic;
		PS2_Data : IN std_logic;
		DoRead : IN std_logic;    
		Trigger : INOUT std_logic;      
		Scan_Code : OUT std_logic_vector(7 downto 0);
		Scan_ready : OUT std_logic
		);
	END COMPONENT;	
	SIGNAL DoRead :      std_logic;    
	SIGNAL Trigger :     std_logic;      
	SIGNAL Scan_Code :   std_logic_vector(7 downto 0);
	SIGNAL Scan_ready :  std_logic;
		
	SIGNAL Ax,Bx: 			std_logic; 
BEGIN
   PB_Sys: PicoBlaze_System1 PORT MAP(
		Clk, write_strobe, read_strobe,
		interrupt_ack, interrupt,
		in_port, port_id, out_port);
   
	Blank <= "0000";
	Mux_Display: Muxdisplay_ver5f PORT MAP(
	    Clk, Blank, Hex, Dp, Segm, An, Clk_1kHz, Clk_1Hz);
		 
   Timer_Sys: Timer PORT MAP(
		Clk,interrupt_ack,T_Scale,interrupt);
		
   VGA_screen: VGA_system PORT MAP(
		Reset 	=> Reset, 
		Clk 		=> Clk, 
		Data_out => Data_From_RAM, 
		VGA_Plug => VGA_Plug, 
		Addr 		=> Addr, 
		Data_in 	=> Data_To_RAM, 
		WE 		=> WE 
   );
	
	PS2_Keyboard: ps2_ctrl_VHDL PORT MAP(
		Clk 			=> Clk,
		Reset 		=> Reset,
		PS2_Clk 		=> PS2_Clk,
		PS2_Data 	=> PS2_Data,
		Scan_Code 	=> Scan_Code,
		Scan_ready 	=> Scan_ready,
		Trigger 		=> Trigger,
		DoRead 		=> DoRead 	
	);		
	
	Temp_Line <= Line + Line_offset;
	Addr      <= Temp_Line & Pixel; 
	Reset     <= '0';
   DoRead    <= '1';

--##############################################################################
  rotary_filter: process(clk)
	   variable BA: std_logic_vector( 1 downto 0);
	begin
		if clk'event and clk='1' then
		   BA := JA(2)&JA(1);
			case BA is
				when "00" => 	Ax <= '0';
				when "01" => 	Bx <= '0';
				when "10" => 	Bx <= '1';
				when "11" => 	Ax <= '1';
				when others => Null;
			end case;
		end if;
   end process rotary_filter;
	
--##############################################################################
--# KCPSM3 input ports 
--##############################################################################
  input_ports: process(clk)
  begin
     if clk'event and clk='1' then
--	     if Read_strobe='1' then
			  Case conv_integer( Port_id(7 downto 0)) is
					when  0 => in_port <= "0000"& Hex(  3 downto  0);
					when  1 => in_port <= "0000"& Hex(  7 downto  4);
					when  2 => in_port <= "0000"& Hex( 11 downto  8);
					when  3 => in_port <= "0000"& Hex( 15 downto 12);
					when  4 => in_port <= Hex( 7 downto 0);
					when  5 => in_port <= Hex(15 downto 8);
					when  6 => in_port <= "0000" & Dp;
					when  7 => in_port <= T_Scale;
					when  9 => in_port <=  Led;						
				  ------------------------------------------------------------
					when 20 => in_port <=  Sw;
					when 21 => in_port <=  "0000"& Btn;
					when 22 => in_port <=  "00"&JA(3 to 4)&"00"&JA(1 to 2);
					when 23 => in_port <=  "00"&JA(3 to 4)&"00"&Ax&Bx;
				  ------------------------------------------------------------
				   when 24 => in_port <= Scan_Code;
					when 25 => in_port <= PS2_Data&PS2_Clk&"00000"&Scan_Ready;
				  ------------------------------------------------------------
					when 32 => in_port <= "0000"&Data_From_RAM;
					when 33 => in_port <= Line;
					when 34 => in_port <= Pixel;			
					when others => null;
			  end case;
--		  end if;
	  end if;
  end process input_ports;

--##############################################################################
--# KCPSM3 output ports 
--##############################################################################
  output_ports: process(clk, Port_id, out_port)
  begin
     if clk'event and clk='1' then
			if write_strobe='1' then
			   Line_Offset <= "00000000";
				Case conv_integer( Port_id(7 downto 0)) is
					when  0 => Hex( 3 downto  0) <= out_port(3 downto 0);
					when  1 => Hex( 7 downto  4) <= out_port(3 downto 0);
					when  2 => Hex(11 downto  8) <= out_port(3 downto 0);
					when  3 => Hex(15 downto 12) <= out_port(3 downto 0);
					when  4 => Hex( 7 downto  0) <= out_port;
					when  5 => Hex(15 downto  8) <= out_port;
					when  6 => Dp                <= out_port(3 downto 0);
					when  7 => T_Scale   			<= out_port;
					when  9 => Led 		         <= out_port;
				 -----------------------------------------------------------------------------
             ------------ VideoRAM(Line,Pixel) = Data   
					when 32 => Data_To_RAM 		   <= out_port(3 downto 0);
					           WE   <= '1'; 
				 ------------ Set Line #
					when 33 => Line 			   	<= out_port(7 downto 0);	
				 ------------ Set Pixel #					
					when 34 => Pixel 			   	<= out_port(7 downto 0);
             ------------ VideoRAM(Line,Pixel) = Data / Line+1					
					when 35 => Data_To_RAM 		   <= out_port(3 downto 0);
					           WE   <= '1'; 
								  IF Line>=240 THEN          --LF                    
								     Line <= (others=>'0');
								  ELSE
									  Line <= Line+1;
								  END IF; 
             ------------ VideoRAM(Line,Pixel) = Data / Pixel+1	
					when 36 => Data_To_RAM 		   <= out_port(3 downto 0);
					           WE   <= '1'; 
								  IF Pixel>=255 THEN 			--NextPixel
                             Pixel <= (others=>'0');
                          ELSE									  
                             Pixel <= Pixel+1;	
								  END IF;
             ------------ VideoRAM(Line,Pixel) = Data / (Line:Pixel)+1									  
					when 37 => Data_To_RAM 		   <= out_port(3 downto 0);
					           WE   <= '1'; 
								  IF Pixel>=255 THEN  			-- CR/LF
                             Pixel <= (others=>'0');
									  IF Line>=240 THEN 
										  Line <= (others=>'0');
									  ELSE
										  Line <= Line+1;
									  END IF;									 
                          ELSE									  
                             Pixel <= Pixel+1;	
								  END IF;
             ------------ VideoRAM(Line+Offset,Pixel) = Data 							  
					when 38 => -- NOTE! The Sign-extension applied to the offset                           
								  Line_offset <= out_port(7)&out_port(7)&out_port(7)&
								                 out_port(7)& OUT_PORT(7 downto 4);
								  Data_To_RAM <= out_port(3 downto 0);
								  WE<='1'; 
             ------------ (Line:Pixel)+16/1											  
					when 39 => WE<='0';                           	
                          IF Pixel>=255 THEN 
                             Pixel <= (others=>'0');
									  IF Line>=239 THEN 
										  Line <= (others=>'0');
									  ELSE
										  Line <= Line+16;
									  END IF;									 
                          ELSE									  
                             Pixel <= Pixel+1;	
								  END IF;	
             ------------ (Line:Pixel)+5/1										  
					when 40 => WE<='0';                           	
                          IF Pixel>=255 THEN 
                             Pixel <= (others=>'0');
									  IF Line>=239 THEN 
										  Line <= (others=>'0');
									  ELSE
										  Line <= Line+5;
									  END IF;									 
                          ELSE									  
                             Pixel <= Pixel+1;	
								  END IF;								  
					when others => null;
				end case;
			end if;

     end if;
--	  if conv_integer( Port_id(7 downto 0))=38 then
--		  Line_offset <= out_port(7)&out_port(7)&out_port(7)&
--						   out_port(7)& OUT_PORT(7 downto 4);
--	  else
--		  Line_offset <= "00000000";
--	  end if;
  end process output_ports;		 
end Behavioral;

