----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity VGA_ColorMap is
Port (CLK :			in  std_logic;
	   COLOR:		in  std_logic_vector ( 3 downto 0 );
		VSYNCH_IN:	in  std_logic;
		HSYNCH_IN:	in  std_logic;
	   HBLANK: 		in  std_logic; 
      VGA_plug:	out Std_logic_vector(9 downto 0));
end VGA_ColorMap;

architecture Behavioral of VGA_ColorMap is
	signal VSYNCH: Std_logic;
	signal HSYNCH: Std_logic;
	signal RGB:    Std_logic_vector( 5 downto 0);
begin

	Pipeline: process	 (CLK)
	begin
		if ( CLK = '1' and CLK'event ) then
			VSYNCH  <= VSYNCH_IN;
			HSYNCH  <= HSYNCH_IN;
		end if;
	end process;
	
	Colour_Map: process  (CLK)
	begin
		if ( CLK = '1' and CLK'event ) then
		   if HBLANK='1' THEN
			   RGB <= "000000";
			else	
				case COLOR is
						when "0000" =>	 RGB <= "000000";  --       Black
						when "0001" =>	 RGB <= "000001";  -- Light Blue
						when "0010" =>	 RGB <= "000100";  -- Light Green
						when "0011" =>	 RGB <= "000101";  -- Light Cyan   
						when "0100" =>	 RGB <= "010000";  -- Light Red
						when "0101" =>	 RGB <= "010001";  -- Light Magenta    
						when "0110" =>	 RGB <= "010100";  -- Light Yellow     
						when "0111" =>	 RGB <= "010101";	  --       Gray
						------------------------------------------------------
						when "1000" =>	 RGB <= "001100"; -- Test ...
						when "1001" =>	 RGB <= "000011"; -- Blue 
						when "1010" =>	 RGB <= "001100"; -- Green
						when "1011" =>	 RGB <= "001111"; -- Cyan   
						when "1100" =>	 RGB <= "110000"; -- Red
						when "1101" =>	 RGB <= "110011"; -- Magenta
						when "1110" =>	 RGB <= "111100"; -- Yellow 
						when "1111" =>	 RGB <= "111111"; -- White
						
						when others =>  RGB <= "111111";
					end case;
         end if;					
		end if;
	end process;
             
   VGA_Plug <= VSYNCH & HSYNCH         -- Synch
	          & RGB(3) & RGB(2) & '0'  -- Green 
				 & RGB(1) & RGB(0)         -- Blue   
				 & RGB(5) & RGB(4) & '0';  -- Red 
end Behavioral;

