
----------------------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ps2_ctrl_VHDL is
    Port ( Clk :        in  STD_LOGIC;
           Reset :      in  STD_LOGIC;
           PS2_Clk :    in  STD_LOGIC;
           PS2_Data :   in  STD_LOGIC;
           Scan_Code :  out  STD_LOGIC_VECTOR (7 downto 0);
           Scan_ready : out  STD_LOGIC;
           Trigger :    inout  STD_LOGIC;
           DoRead :     in STD_LOGIC);
end ps2_ctrl_VHDL;

architecture Behavioral of ps2_ctrl_VHDL is
	signal S_reg:  STD_LOGIC_VECTOR (7 downto 0);
	-- After start bit  =  1  0  0  0  0  0  0  0
   -- After Data bit 0 = d0  1  0  0  0  0  0  0
	-- After Data bit 1 = d1 d0  1  0  0  0  0  0
	--
	-- After Data bit 6 = d6 d5 d4 d3 d2 d1 d0  1 <- This 1 bit indicates data almost ready
	-- After Data bit 7 = d7 d6 d5 d4 d3 d2 d1 d0    Time to change state to Parity check
begin

   This_Process_Synchronize_Data_exchange:	
	process( Clk)
	   Variable Clear_reg:  STD_LOGIC_VECTOR (1 downto 0);
	begin
	   if rising_edge( clk) then
		   if Reset='1' then
			   Scan_code  <= "00000000";
				Scan_Ready <= '0';
				Clear_reg  := "00";
			else
				if (trigger='1') and (DoRead='1') then   
						Scan_Code  <= S_reg;	    --Scan_Code gets the shifted in value of PS2_Data 
						Clear_reg  := "00";
						scan_ready <=	'1';		 --New scan code has been received
				else
					clear_reg := clear_reg + 1; --Wait two clock cycles before clearing scan_ready
					if (clear_reg >= "10") then
						Clear_reg  := "00";
						scan_ready <=	'0';
					end if;
				end if;			
			end if;
		end if;
	end process This_Process_Synchronize_Data_exchange;

	----------------------------------------------------------------------------------
	--                     start    b0   8 Data-bits     b7  parity   stop     
	-- PS2_Data = -------__________----=================------ ppppppp ssssss ----
	-- PS2_Clk  = -----------____----____---...........---___----____----___
	--
	----------------------------------------------------------------------------------
	This_Process_Recieve_Data_from_the_PS2:
	process( PS2_Clk, Reset)
	   type     States is (Start_bit, Data_bits, Parity_bit, Stop_bit);
		variable Read_state: States;
   begin
	   if Reset='1' then
			Read_State := Start_Bit;
			S_Reg      <= "10000000";    -- The 1-bit acts as a "Counter"
	   elsif falling_edge( PS2_Clk) then
			Trigger <= '0';
			case Read_State is
			
				when Start_bit => 
								 if PS2_Data='0' then
									 S_Reg  <= "10000000";
									 Read_State := Data_Bits;
								 end if;
								 
				when Data_bits => 
								 if S_Reg(0)='1' then    -- Change state after 8 data bits
									 Read_State := Parity_bit;
								 end if;  
								 S_Reg <= PS2_Data & S_reg(7 downto 1); -- Next data-bit
								 
				when Parity_bit =>                      -- not used or checked
								 Read_State := Stop_bit;  	
								 
				when Stop_bit  =>                       -- not checked
								 Trigger    <= '1';         -- Indicates data ready now
								 Read_State := Start_bit;   -- One more time
			end case;
		end if;
	end process This_Process_Recieve_Data_from_the_PS2;
	
end Behavioral;

